class SeverityLevel(object):
    """Data contract class for type SeverityLevel."""
    # Enumeration value verbose
    verbose = 0

    # Enumeration value information
    information = 1

    # Enumeration value warning
    warning = 2

    # Enumeration value error
    error = 3

    # Enumeration value critical
    critical = 4
